﻿Public Class Form1
    Private m_Refresh As Boolean
    Private m_Bm As Bitmap

    Private Sub btnGo_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnGo.Click
        Dim depth As Integer = Integer.Parse(txtDepth.Text)
        If (depth > 8) Then
            If (MessageBox.Show("A large depth may take a long time to draw. Do you want to continue?", _
                "Continue?", MessageBoxButtons.YesNo, _
                MessageBoxIcon.Question) = DialogResult.No) _
            Then
                Return
            End If
        End If

        Me.Cursor = Cursors.WaitCursor
        Application.DoEvents()

        ' See if we should refresh as we draw.
        m_Refresh = chkRefresh.Checked

        m_Bm = New Bitmap(picCanvas.ClientSize.Width, picCanvas.ClientSize.Height)
        picCanvas.Image = m_Bm

        Using gr As Graphics = Graphics.FromImage(m_Bm)
            ' Draw the curve.
            gr.Clear(picCanvas.BackColor)

            Dim dx As Single = CSng(m_Bm.Width / Math.Pow(2, depth - 1) / 8)
            Dim dy As Single = CSng(m_Bm.Height / Math.Pow(2, depth - 1) / 8)
            Sierpinski(gr, depth, dx, dy)
        End Using

        ' Display the result.
        picCanvas.Refresh()
        Me.Cursor = Cursors.Default
    End Sub

    ' Draw a Sierpinski curve.
    Private Sub Sierpinski(ByVal gr As Graphics, ByVal depth As Integer, ByVal dx As Single, ByVal dy As Single)
        Dim x As Single = 2 * dx
        Dim y As Single = dy

        SierpA(gr, depth, dx, dy, x, y)
        DrawRel(gr, x, y, dx, dy)
        SierpB(gr, depth, dx, dy, x, y)
        DrawRel(gr, x, y, -dx, dy)
        SierpC(gr, depth, dx, dy, x, y)
        DrawRel(gr, x, y, -dx, -dy)
        SierpD(gr, depth, dx, dy, x, y)
        DrawRel(gr, x, y, dx, -dy)

        picCanvas.Refresh()
    End Sub

    ' Draw right across the top.
    Private Sub SierpA(ByVal gr As Graphics, ByVal depth As Integer, ByVal dx As Single, ByVal dy As Single, ByRef x As Single, ByRef y As Single)
        If (depth > 0) Then
            depth -= 1

            SierpA(gr, depth, dx, dy, x, y)
            DrawRel(gr, x, y, dx, dy)
            SierpB(gr, depth, dx, dy, x, y)
            DrawRel(gr, x, y, 2 * dx, 0)
            SierpD(gr, depth, dx, dy, x, y)
            DrawRel(gr, x, y, dx, -dy)
            SierpA(gr, depth, dx, dy, x, y)
        End If

        If (m_Refresh) Then picCanvas.Refresh()
    End Sub

    ' Draw down on the right.
    Private Sub SierpB(ByVal gr As Graphics, ByVal depth As Integer, ByVal dx As Single, ByVal dy As Single, ByRef x As Single, ByRef y As Single)
        If (depth > 0) Then
            depth -= 1
            SierpB(gr, depth, dx, dy, x, y)
            DrawRel(gr, x, y, -dx, dy)
            SierpC(gr, depth, dx, dy, x, y)
            DrawRel(gr, x, y, 0, 2 * dy)
            SierpA(gr, depth, dx, dy, x, y)
            DrawRel(gr, x, y, dx, dy)
            SierpB(gr, depth, dx, dy, x, y)
        End If

        If (m_Refresh) Then picCanvas.Refresh()
    End Sub

    ' Draw left across the bottom.
    Private Sub SierpC(ByVal gr As Graphics, ByVal depth As Integer, ByVal dx As Single, ByVal dy As Single, ByRef x As Single, ByRef y As Single)
        If (depth > 0) Then
            depth -= 1
            SierpC(gr, depth, dx, dy, x, y)
            DrawRel(gr, x, y, -dx, -dy)
            SierpD(gr, depth, dx, dy, x, y)
            DrawRel(gr, x, y, -2 * dx, 0)
            SierpB(gr, depth, dx, dy, x, y)
            DrawRel(gr, x, y, -dx, dy)
            SierpC(gr, depth, dx, dy, x, y)
        End If

        If (m_Refresh) Then picCanvas.Refresh()
    End Sub

    ' Draw up along the left.
    Private Sub SierpD(ByVal gr As Graphics, ByVal depth As Integer, ByVal dx As Single, ByVal dy As Single, ByRef x As Single, ByRef y As Single)
        If (depth > 0) Then
            depth -= 1
            SierpD(gr, depth, dx, dy, x, y)
            DrawRel(gr, x, y, dx, -dy)
            SierpA(gr, depth, dx, dy, x, y)
            DrawRel(gr, x, y, 0, -2 * dy)
            SierpC(gr, depth, dx, dy, x, y)
            DrawRel(gr, x, y, -dx, -dy)
            SierpD(gr, depth, dx, dy, x, y)
        End If

        If (m_Refresh) Then picCanvas.Refresh()
    End Sub

    ' Draw a line between (x, y) and (x + dx, y + dy).
    ' Update x and y.
    Private Sub DrawRel(ByVal gr As Graphics, ByRef x As Single, ByRef y As Single, ByVal dx As Single, ByVal dy As Single)
        gr.DrawLine(Pens.Black, x, y, x + dx, y + dy)
        x += dx
        y += dy
    End Sub
End Class
